import os
import shutil
import subprocess
import typing


def is_skopeo() -> bool :
    return shutil.which("skopeo") is not None


def skopeo_pull(base_image: str, dest: typing.Union[str, os.PathLike], params: typing.Sequence[str]=()):
    if isinstance(dest, os.PathLike):
        dest = str(dest)
    return subprocess.run(["skopeo", "copy", "--multi-arch", "all", *params, "--remove-signatures", "docker://"+base_image, "oci:"+dest+":latest"], check=True)


def skopeo_push(src: typing.Union[str, os.PathLike], oci_ref: str, params: typing.Sequence[str]=()):
    if isinstance(src, os.PathLike):
        src = str(src)
    return subprocess.run(["skopeo", "copy", "--multi-arch", "all", *params, "oci:"+src+":latest", "docker://"+oci_ref], check=True)


def skopeo_inspect(skopeo_ref: str, params: typing.Sequence[str]=()) -> str:
    """Use skopeo inspect --raw (<params...>) <skopeo_ref>
    To use this function, make sure to respect skopeo's protocols, ie docker://, or oci:<dir>:<tag>, etc
    """
    result = subprocess.run(["skopeo", "inspect", "--raw", *params, skopeo_ref], capture_output=True, text=True, check=True)
    return result.stdout.strip()



import json
from typing import Union
from olot.oci.oci_common import MediaTypes
from olot.oci.oci_image_index import OCIImageIndex
from olot.oci.oci_image_manifest import ContentDescriptor, OCIImageManifest
from olot.utils.types import compute_hash_of_str


def get_descriptor_from_manifest(manifest: str) -> ContentDescriptor:
    """Given a manifest (RAW) return its descriptor
    
    Note: this MUST be the RAW content of the manifest in the repository/oci-layout so to compute the exact size in the descriptor.
    eg. use skopeo inspect --raw or oras manifest fetch (without --pretty)
    """    
    size = len(manifest)
    data = json.loads(manifest)
    model: Union[OCIImageManifest, OCIImageIndex]
    mediaType = data.get("mediaType")
    if mediaType == MediaTypes.manifest:
        model = OCIImageManifest.model_validate_json(manifest)
    elif mediaType == MediaTypes.index:
        model = OCIImageIndex.model_validate_json(manifest)
    else:
        raise ValueError(f"Unsupported manifest type{mediaType}")
    if not model.mediaType:
        raise ValueError(f"Unexpected OCI manifest spec missing MediaType: {model}")
    result = ContentDescriptor(
        mediaType=model.mediaType,
        size=size,
        digest="sha256:" + compute_hash_of_str(manifest),
        artifactType=model.artifactType,
        urls=None,
        data=None,
    )
    return result
